@@ -1,11 +1,13 @@
 #!/usr/bin/env python
-__author__ = ('Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
+__author__ = ('Michal Puncochar (michal.puncochar@unitn.it), '
+              'Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
               'Duy Tin Truong (duytin.truong@unitn.it), '
               'Francesco Asnicar (f.asnicar@unitn.it), '
               'Moreno Zolfo (moreno.zolfo@unitn.it), '
               'Francesco Beghini (francesco.beghini@unitn.it)')
-__version__ = '4.0.3'
-__date__ = '24 Oct 2022'
+__version__ = '4.0.4'
+__date__ = '17 Jan 2023'
+
 
 
 import os
@@ -15,8 +17,12 @@ import tempfile
 import bz2
 import pickletools
 import argparse as ap
-from cmseq import cmseq
+from collections import Counter
 from shutil import rmtree
+
+import pysam
+import scipy.stats as sps
+
 try:
     from .external_exec import samtools_sam_to_bam, samtools_sort_bam_v1, decompress_bz2
     from .util_fun import info, error
@@ -34,6 +40,15 @@ except ImportError:
 class SampleToMarkers:
     """SampleToMarkers class"""
 
+    class DEFAULTS:
+        pileup_stepper = 'nofilter'
+        poly_error_rate = 0.001
+        min_base_coverage = 1
+        min_base_quality = 30
+        min_mapping_quality = 10
+        poly_dominant_frq_thrsh = 0.8
+        poly_pvalue_threshold = 0.05
+
     def decompress_from_bz2(self):
         """ Decompressed SAM.BZ2 files
 
@@ -41,7 +56,7 @@ class SampleToMarkers:
             (list, str): tuple with the list of decompressed files and their format
         """
         results = execute_pool(
-            ((self.decompress_bz2_file, i, self.tmp_dir) for i in self.input), self.nprocs)
+            ((SampleToMarkers.decompress_bz2_file, i, self.tmp_dir) for i in self.input), self.nprocs)
         decompressed = [r[0] for r in results]
         decompressed_format = [r[1] for r in results]
         if decompressed_format[1:] == decompressed_format[:-1]:
@@ -54,6 +69,7 @@ class SampleToMarkers:
         else:
             error("Decompressed files have different formats", exit=True)
 
+    @staticmethod
     def decompress_bz2_file(input_file, output_dir):
         """Decompress a BZ2 file and returns the decompressed file and the decompressed file format
 
@@ -82,12 +98,16 @@ class SampleToMarkers:
             self.input = execute_pool(
                 ((samtools_sort_bam_v1, i, self.tmp_dir) for i in self.input), self.nprocs)
             info("\tDone.")
-        elif self.sorted == False:
+        elif not self.sorted:
             info("\tSorting BAM samples...")
             self.input = execute_pool(
                 ((samtools_sort_bam_v1, i, self.tmp_dir) for i in self.input), self.nprocs)
             info("\tDone.")
 
+        info('\tIndexing BAM samples...')
+        execute_pool(((pysam.index, i) for i in self.input), self.nprocs)
+        info('\tDone.')
+
     def build_consensus_markers(self, filtered):
         """Gets the markers for each sample and writes the Pickle files
 
@@ -96,24 +116,65 @@ class SampleToMarkers:
         """
         for i in self.input:
             info("\tProcessing sample: {}".format(i))
-            results = self.execute_cmseq(i)
+            results = self.get_consensuses_for_sample(i)
             self.write_results_as_pkl(filtered, os.path.splitext(
                 os.path.basename(i))[0], results)
             info("\tDone.")
 
-    def execute_cmseq(self, input_bam):
-        """cmseq on the specified BAM file
+
+    def get_consensuses_for_sample(self, input_bam, stepper=DEFAULTS.pileup_stepper,
+                                   error_rate=DEFAULTS.poly_error_rate,
+                                   poly_pvalue_threshold=DEFAULTS.poly_pvalue_threshold):
+        """Pileup on the specified BAM file
 
         Args:
             input_bam (str): the path to the input BAM file
+            stepper (str): stepper to use in the pileup
+            error_rate (float): the sequencing error rate
+            poly_pvalue_threshold (float): the p-value threshold to call polymorphic position
 
         Returns:
-            list: the list with the reconstructed consensus markers
+            list[tuple[str, str]]: the list with the marker names and consensus sequences
         """
-        collection = cmseq.BamFile(
-            input_bam, index=True, minlen=self.min_read_len, minimumReadsAligning=self.min_reads_aligning)
-        return collection.parallel_reference_free_consensus(ncores=self.nprocs, mincov=self.min_base_coverage, minqual=self.min_base_quality,
-                                                            consensus_rule=cmseq.BamContig.majority_rule_polymorphicLoci, dominant_frq_thrsh=self.dominant_frq_threshold)
+
+        info('\t\tLoading the bam file and extracting information...')
+        sam_file = pysam.AlignmentFile(input_bam)
+
+        all_markers = sam_file.references
+        marker_lengths = sam_file.lengths
+        marker_to_length = dict(zip(all_markers, marker_lengths))
+        info('\t\tDone.')
+
+        ACTGactg = set(list('ACTGactg'))
+
+        info('\t\tRunning the pileup...')
+        consensuses = {m: bytearray(b'-' * marker_to_length[m]) for m in all_markers}
+        for base_pileup in sam_file.pileup(contig=None, stepper=stepper, min_base_quality=self.min_base_quality):
+            marker = base_pileup.reference_name
+            pos = base_pileup.pos
+            bases = [b.upper() for b in base_pileup.get_query_sequences() if b in ACTGactg]
+            base_coverage = len(bases)
+            if base_coverage < self.min_base_coverage:
+                continue
+
+            base_frequencies = Counter(bases)
+            max_frequency = max(base_frequencies.values())
+            max_ratio = max_frequency / base_coverage
+            base_consensus = base_frequencies.most_common(1)[0][0]
+
+            if max_ratio < self.dominant_frq_threshold:
+                p_value = sps.binom.cdf(max_frequency, base_coverage, 1 - error_rate)
+                if p_value <= poly_pvalue_threshold:
+                    base_consensus = '*'  # mask out likely polymorphic positions
+
+            consensuses[marker][pos] = ord(base_consensus)
+
+        info('\t\tDone.')
+
+        consensuses = [(m, c.decode()) for m, c in consensuses.items()]  # convert bytearrays to strings
+        return consensuses
+
+
 
     def write_results_as_pkl(self, filtered, name, results):
         """Writes the consensus sequences as a PKL file
@@ -134,11 +195,15 @@ class SampleToMarkers:
                 pickle.dumps(consensus, pickle.HIGHEST_PROTOCOL)))
 
     def parallel_filter_sam(self, input_file, filtered_markers):
-        """Filters an input SAM file with the hits against markers of specific clades
+        """
+        Filters an input SAM file
+            * filters out viral markers (VDB)
+            * filters out hits with low mapQ (argument --min_mapping_quality)
+            * filters out markers with not enough mapped reads (argument --min_reads_aligning)
 
         Args:
             input_file (str): the input SAM file
-            filtered_markers (list): the list with the markers of the filtered clades
+            filtered_markers (set): the list with the markers of the filtered clades, None if to use all markers
 
         Returns:
             str: the path to the output file
@@ -147,22 +212,46 @@ class SampleToMarkers:
         if self.input_format.lower() == "bz2":
             ifn = bz2.open(input_file, 'rt')
             output_file = output_file.replace('.bz2', '')
-        elif self.input_format.lower() == "sam":
+        else:
+            assert self.input_format.lower() == "sam"
             ifn = open(input_file, 'r')
+
+        def filter_mapping_line(line_fields_, markers_subset):
+            marker_ = line_fields_[2]
+            if marker_.startswith('VDB'):
+                return False
+            if markers_subset is not None and marker_ not in markers_subset:
+                return False
+            mapq_ = int(line_fields_[4])
+            if mapq_ < self.min_mapping_quality:
+                return False
+            return True
+
+        marker_to_reads = Counter()
+        for line in ifn:
+            if line.startswith('@'):
+                continue
+            line_fields = line.rstrip('\n').split('\t')
+            if filter_mapping_line(line_fields, filtered_markers):
+                marker = line_fields[2]
+                marker_to_reads[marker] += 1
+
+        selected_markers = set((m for m, c in marker_to_reads.items() if c >= self.min_reads_aligning))
+
+        ifn.seek(0)  # second pass of the file
         with open(output_file, 'w') as ofn:
             for line in ifn:
-                s_line = line.strip().split('\t')
-                if line.startswith('@'):
-                    if line.startswith('@HD'):
-                        ofn.write(line)
-                    elif filtered_markers is not None and s_line[1].split(':')[1] in filtered_markers:
-                        ofn.write(line)
-                    elif filtered_markers is None and not s_line[1].split(':')[1].startswith('VDB'):
-                        ofn.write(line)
-                elif filtered_markers is not None and s_line[2] in filtered_markers:
-                    ofn.write(line)
-                elif filtered_markers is None and not s_line[2].startswith('VDB'):
+                line_fields = line.rstrip('\n').split('\t')
+                if line.startswith('@HD'):
                     ofn.write(line)
+                elif line.startswith('@'):
+                    marker = line_fields[1].split(':')[1]
+                    if marker in selected_markers:
+                        ofn.write(line)
+                else:
+                    if filter_mapping_line(line_fields, selected_markers):
+                        ofn.write(line)
+
         ifn.close()
         return output_file
 
@@ -193,7 +282,7 @@ class SampleToMarkers:
         info("Done.")
         if not self.debug:
             info("Removing temporary files...")
-            rmtree(self.tmp_dir, ignore_errors=False, onerror=None)
+            rmtree(self.tmp_dir)
             info("Done.")
 
     def __init__(self, args):
@@ -204,9 +293,9 @@ class SampleToMarkers:
         self.database_controller = MetaphlanDatabaseController(args.database)
         self.breadth_threshold = args.breadth_threshold
         self.min_reads_aligning = args.min_reads_aligning
-        self.min_read_len = args.min_read_len
         self.min_base_coverage = args.min_base_coverage
         self.min_base_quality = args.min_base_quality
+        self.min_mapping_quality = args.min_mapping_quality
         self.dominant_frq_threshold = args.dominant_frq_threshold
         self.clades = args.clades
         self.tmp_dir = args.output_dir if args.tmp is None else args.tmp
@@ -237,20 +326,21 @@ def read_params():
                    help="The breadth of coverage threshold for the consensus markers")
     p.add_argument('--min_reads_aligning', type=int, default=8,
                    help="The minimum number of reads to cover a marker")
-    p.add_argument('--min_read_len', type=int, default=cmseq.CMSEQ_DEFAULTS.minlen,
-                   help="The minimum lenght for a read to be considered")
-    p.add_argument('--min_base_coverage', type=int, default=cmseq.CMSEQ_DEFAULTS.mincov,
+    p.add_argument('--min_base_coverage', type=int, default=SampleToMarkers.DEFAULTS.min_base_coverage,
                    help="The minimum depth of coverage for a base to be considered")
-    p.add_argument('--min_base_quality', type=int, default=cmseq.CMSEQ_DEFAULTS.minqual,
+    p.add_argument('--min_base_quality', type=int, default=SampleToMarkers.DEFAULTS.min_base_quality,
                    help="The minimum quality for a base to be considered. This is performed BEFORE --min_base_coverage")
-    p.add_argument('--dominant_frq_threshold', type=float, default=cmseq.CMSEQ_DEFAULTS.poly_dominant_frq_thrsh,
+    p.add_argument('--min_mapping_quality', type=int, default=SampleToMarkers.DEFAULTS.min_mapping_quality,
+                   help="The minimum quality for a mapping of the read to be considered.")
+    p.add_argument('--dominant_frq_threshold', type=float, default=SampleToMarkers.DEFAULTS.poly_dominant_frq_thrsh,
                    help="The cutoff for degree of 'allele dominance' for a position to be considered polymorphic")
     p.add_argument('--clades', type=str, nargs='+', default=[],
                    help="Restricts the reconstruction of the markers to the specified clades")
     p.add_argument('--tmp', type=str, default=None,
                    help="If specified, the directory where to store the temporal files")
     p.add_argument('--debug', action='store_true', default=False,
-                   help="If specified, StrainPhlAn will not remove the temporal folder. Not available with inputs in BAM format")
+                   help="If specified, StrainPhlAn will not remove the temporal folder. "
+                        "Not available with inputs in BAM format")
     p.add_argument('-n', '--nprocs', type=int, default=1,
                    help="The number of threads to execute the script")
     return p.parse_args()
@@ -268,7 +358,7 @@ def check_params(args):
         error('-f (or --input_format) must be specified', exit=True)
     elif not args.output_dir:
         error('-o (or --output_dir) must be specified', exit=True)
-    elif args.input_format.lower() != "bam" and args.input_format.lower() != "sam" and args.input_format.lower() != "bz2":
+    elif args.input_format.lower() not in ['bam', 'sam', 'bz2']:
         error('The input format must be SAM, BAM, or compressed in BZ2 format', exit=True)
     elif args.input_format.lower() == "bam" and len(args.clades) > 0:
         error('The --clades option cannot be used with inputs in BAM format', exit=True)
